;***************************************************************************
;*                                                                         *
;* ACHTUNG:     Programm ist als COM konzipiert                            *
;*              darum mit EXE2BIN in ein  COM-Programm verwandeln          *
;*              oder bei TLINK mit der Option /Tdc linken                  *
;*                                                                         *
;***************************************************************************
;*                                                                         *
;* Datei-Name      : PUTRTC.ASM                                            *
;* Project         : PUTRTC/GETRTC                                         *
;* sonst. Module   : ---                                                   *
;* Compiler        : Assembler                                             *
;* Betriebs-System : MS-DOSE                                               *
;* Autor           : J. Kuenstner                                          *
;* Datum           : Jan.  95                                              *
;* Beschreibung    :                                                       *
;*                                                                         *
;*      Enstand aus dem Artikel: "Stechuhr" in c't 11/94 auf Seite 248     *
;*      Hintergrund-Informationen zum RTC-Chip in c't 12/88 auf Seite 196  *
;*      Zugriff auf die Register im RTC erfolgt folgendermassen:           *
;*      Schreiben der Register-Nummer auf Port 0x70                        *
;*      Lesen oder Schreiben des gewuenschten Registers auf Port 0x71      *
;*                                                                         *
;* Programm merkt sich in Alarm-Registern der RTC die aktuelle Zeit.       *
;* Diese Zeit wird alle 10 s aktualisiert.                                 *
;* Durch Auslesen der Alarm-Register vor erneutem Start dieses Programms   *
;* ist es so moeglich die Ausschaltzeit des PC's zu bestimmen.             *
;*                                                                         *
;*        RTC-Register                                                     *
;*       Ŀ                                       *
;*        0:  Sekunde                                                   *
;*        1:  Alarm-Sekunde                                             *
;*        2:  Minute                                                    *
;*        3:  Alarm-Minute                                              *
;*        4:  Stunde                                                    *
;*        5:  Alarm-Stunde                                              *
;*        6:  Wochentag                                                 *
;*        7:  Tag                                                       *
;*        8:  Monat                                                     *
;*        9:  Jahr                                                      *
;*       10:  StatusRegister A                                          *
;*       11:  StatusRegister B                                          *
;*       12:  StatusRegister C                                          *
;*       13:  StatusRegister D                                          *
;*                                              *
;*                                                                         *
;*                                                                         *
;*                                                                         *
;*                                                                         *
;***************************************************************************
;*                               HISTORY                                   *
;*                                                                         *
;* Datum     | wer         | was                                           *
;* ------------------------------------------------------------------------*
;*           |             |                                               *
;* 28.01.95  | Jogi        | Erst-Implementierung                          *
;* ------------------------------------------------------------------------*
;* 20.02.95  | Jogi        | Simpler Test auf Doppelaufruf                 *
;* ------------------------------------------------------------------------*
;* 28.02.98  | Jogi        | Ein paar Kommentare rein, ich leg das Dingens *
;*           |             | auf meine Home-Page                           *
;*           |             |                                               *
;*                                                                         *
;***************************************************************************




DOS             EQU 21h         ; DOS-Interrupts

TICKER_INT      EQU 1Ch         ; Dos-Ticker-Interrupt

ZEICHENAUSG     EQU 02          ; DL=ASCII-CODE
STRINGAUSG      EQU 09h         ; DOS-Fkt. 09h DS:DX auf String
GETVECT         EQU 35h         ; DOS-Fkt. 35h getvect
SETVECT         EQU 25h         ; DOS-Fkt. 25h setvect
TESTKEYB        EQU 0Bh         ; DOS-Fkt. 0Bh Status eingabe testen
RESI_ENDE       EQU 31h         ; DOS-Fkt. 31h speicherresident beenden
NORM_ENDE       EQU 04Ch        ; DOS-Fkt. 4Ch normal beenden

RTC_CHOICE      EQU 70h         ; Auswahl was gelesen werden soll
RTC_REG         EQU 71h         ; das eigentliche Schreib-Lese-Register

SEK             EQU 0
AL_SEK          EQU 1
MIN             EQU 2
AL_MIN          EQU 3
HOUR            EQU 4
AL_HOUR         EQU 5
DAY             EQU 7
STATUS_A        EQU 10




;===========================================================================
;


code segment
        assume cs:code, ds:code


org     100h

;---------------------------------------------------------------------------
; Normaler Startup-Code

start:  jmp     init


;---------------------------------------------------------------------------
; Zwei Variablen:

	count dw 0			; Der 10-Sekunden-Zaehler
	old_vector  dd ?		; und der alte Ticker


;===========================================================================
; Meine Interrupt-Routine fuer den Ticker !

my_ticker proc far

	pushf				; fuer nachfolgende Routine
	push	ax
	push	bx
	push	ds

	mov	ax,cs			; auf alte Register umstellen
	mov	ds,ax			;

;---------------------------------------------------------------------------
; Sind schon 10 Sekunden vorbei ?

	inc	count			; den Counter erhoehen
	cmp	count,182		; schon 10 Sekunden vorbei ?
	jl	sz_my_ticker_end	; nein, dann wieder raus

	mov	count,0			; Counter-Reset

;---------------------------------------------------------------------------
; Wird der RTC-Baustein gerade upgedatet ?

	mov	al,STATUS_A		; Wahl-Register
	out	RTC_CHOICE,al		; auf Status_A

	in	al,RTC_REG		; das Register einlesen
	shl	al,1			; MSB ins Carry-Flag
	jc	sz_my_ticker_end	; wenn 1, dann Ende !


;---------------------------------------------------------------------------
; Nein, Baustein wird nicht upgdatet, also Alarm-Zeit = Zeit

	cli				; Interrupts sperren weil
					; nix reinhauen in lesen


;---------------------------------------------------------------------------
; Minuten

	mov	al,MIN			;
	out	RTC_CHOICE,al		; Wahl-Register auf Minuten
	in	al,RTC_REG		; die Minuten einlesen

	mov	bl,al			; sichern

	mov	al,AL_MIN		; Wahl-Register
	out	RTC_CHOICE,al		; auf Alarm-MInuten

	mov	al,bl			; und die gespeicherten Minuten
	out	RTC_REG,al		; in die Alarm-Minuten ausgeben

;---------------------------------------------------------------------------
; Stunden

	mov	al,HOUR			;
	out	RTC_CHOICE,al		; Wahl-Register auf Stunden
	in	al,RTC_REG		; die Stunden einlesen

	mov	bl,al			; sichern

	mov	al,AL_HOUR		; Wahl-Register
	out	RTC_CHOICE,al		; auf Alarm-Stunden

	mov	al,bl			; und die gespeicherten Stunden
	out	RTC_REG,al		; in die Alarm-Stunden ausgeben

;---------------------------------------------------------------------------
; Tag:  Achtung in Alarm kein Platz vorgesehen fuer Tag, also
;       Missbrauch von Alarm-Sekunden

	mov	al,DAY			;
	out	RTC_CHOICE,al		; Wahl-Register auf Tag
	in	al,RTC_REG		; den Tag  einlesen

	mov	bl,al			; sichern

	mov	al,AL_SEK		; Wahl-Register
	out	RTC_CHOICE,al		; auf Alarm-Sekunden

	mov	al,bl			; und die gespeicherten Sekunden
	out	RTC_REG,al		; in die Alarm-Sekunden ausgeben


	sti				; Interrupt enable

sz_my_ticker_end:
	pop	ds
	pop	bx
	pop	ax
	popf
	jmp	cs:[old_vector]		; Und den nachfolgenden aufrufen ...


my_ticker endp



meldungen:

meldg	db 'Program PUTRTC V1.1, Idee aus : c',39,'t 11/94',10,13
	db 'Realisierung in Assembler und (c) Jogi 1995-98 ',10,13,'$'
aktiv	db '- ist schon aktiv !',7,10,10,13,'$'

okay	db '- wurde gestartet !',10,13,'$'

init:
	mov	ax,cs			; DatenSegmentRegister laden
	mov	ds,ax			; Standard

;---------------------------------------------------------------------------
; Einschalt-Meldung ausgeben

	mov	dx,offset meldg		; Die Einschalt-Meldung ausgeben
	mov	ah,STRINGAUSG		; ueber DOS-Funktion 09
	int	DOS

;---------------------------------------------------------------------------
; Alten Ticker-Interrupt ermitteln

	mov	ah,GETVECT		; Dos-Funktion Interruptvektor lesen
	mov	al,TICKER_INT		; Ticker-Interrupt-Nr: 1Ch
	int	DOS			; Dos-Funktion aufrufen


;---------------------------------------------------------------------------
; Testen ob schon selbst aktiv

	cmp	bx,offset my_ticker	; Bin ichs etwa selber
	je	sz_ist_aktiv		; ja Meldung an Benutzer ausgeben
					; und terminieren


;---------------------------------------------------------------------------
; Nein , also alten Vektor speichern

	mov	word ptr [old_vector],bx; Alten Vektor sichern
	mov	word ptr [old_vector+2],es


;---------------------------------------------------------------------------
; Eigenen Ticker-Interrupt eintragen

	mov	ah,SETVECT		; Dos-Funktion Interruptvektor setzen
	mov	al,TICKER_INT		; Ticker-Interrupt
	mov	dx,offset my_ticker	; Den Offset der eig. Routine laden
					; normalerweise muss DS auf SEG der
					; Routine eingestellt werden,
					; hier zeigts schon drauf !
	int	DOS			; eigene ISR mit DS:DX installieren



;---------------------------------------------------------------------------
; Fertig, Meldung ausgeben


	mov	dx,offset okay		; Ende der Einschalt-Meldung
	mov	ah,STRINGAUSG		; ausgeben
	int	DOS


;---------------------------------------------------------------------------
; Speicherresident beenden

	mov	dx, offset meldungen	; maximal bis dahin
	mov	cl,4			; aber in Paragraphen
	shr	dx,cl			; also durch 16
	inc	dx			; eine Angst-Zugabe

	mov	ah,RESI_ENDE		; Speicher-Resident
	int	DOS			; beenden


;---------------------------------------------------------------------------
; Wurde schon aufgerufen

sz_ist_aktiv:
	mov	dx,offset aktiv		; Meldung ausgeben
	mov	ah,STRINGAUSG
	int	DOS

sz_norm_ende:				; und normal terminieren
	mov	ah,NORM_ENDE
	int	DOS





code ends
     end start



